==> quetoo-0.3.3.tar.bz2: quetoo-0.3.3/src/vid_ref.c <==
/*
* Copyright(c) 1997-2001 Id Software, Inc.
* Copyright(c) 2002 The Quakeforge Project.
* Copyright(c) 2006 Quetoo.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or(at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
* 
* See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*
* Main windowed and fullscreen graphics interface module. This module
* is used for both the software and OpenGL rendering versions of the
* Quake refresh engine.
*/

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#ifdef HAVE_DLOPEN
# include <dlfcn.h>
#endif

#include "client.h"

// structure containing functions exported from refresh
refexport_t re;

/* Console variables that we need to access from this module */
cvar_t * vid_gamma;      /* gamma value */
cvar_t * vid_ref;        /* name of refresher dll */
cvar_t * vid_xpos;       /* window position x */
cvar_t * vid_ypos;       /* window position y */
cvar_t *vid_width;		/* window size x */
cvar_t *vid_height;		/* window size y */
cvar_t * vid_fullscreen; /* fullscreen on or off */

// global video state; used by other modules
viddef_t viddef;

// handle to refresher
static void * reflib_library = NULL;

qboolean reflib_active = 0;

// keyboard
void Do_Key_Event(int key, qboolean down);

void(*KBD_Update_fp)(void);
void(*KBD_Init_fp)(Key_Event_fp_t fp);
void(*KBD_Close_fp)(void);

// mouse
in_state_t in_state;

void(*REF_IN_Init_fp)(in_state_t *in_state_p);
void(*REF_IN_Shutdown_fp)(void);
void(*REF_IN_Activate_fp)(qboolean active);
void(*REF_IN_Commands_fp)(void);
void(*REF_IN_Move_fp)(usercmd_t *cmd);
void(*REF_IN_Frame_fp)(void);

void Real_IN_Init(void);

// clipboard
char *(*REF_Sys_GetClipboardData_fp)(void);

/*
 
REF GLUE
 
*/

#define MAXPRINTMSG	4096
void VID_Printf(int print_level, char *fmt, ...){
	va_list	argptr;
	char msg[MAXPRINTMSG];
	
	va_start(argptr, fmt);
	vsnprintf(msg, MAXPRINTMSG, fmt, argptr);
	va_end(argptr);
	
	if(print_level == PRINT_ALL)
		Com_Printf("%s", msg);
	else
		Com_DPrintf("%s", msg);
}

void __attribute__((noreturn)) VID_Error(int err_level, char *fmt, ...){
	va_list	argptr;
	char msg[MAXPRINTMSG];
	
	va_start(argptr, fmt);
	vsnprintf(msg, MAXPRINTMSG, fmt, argptr);
	va_end(argptr);
	
	Com_Error(err_level, "%s", msg);
}


/*
** VID_NewWindow
*/
void VID_NewWindow(int width, int height){
	viddef.width = width;
	viddef.height = height;
}

void VID_FreeReflib(void){
	if(reflib_library){
		if(KBD_Close_fp)
			KBD_Close_fp();
		if(REF_IN_Shutdown_fp)
			REF_IN_Shutdown_fp();
		dlclose(reflib_library);
	}
	
	KBD_Init_fp = NULL;
	KBD_Update_fp = NULL;
	KBD_Close_fp = NULL;
	REF_IN_Init_fp = NULL;
	REF_IN_Shutdown_fp = NULL;
	REF_IN_Activate_fp = NULL;
	REF_IN_Commands_fp = NULL;
	REF_IN_Move_fp = NULL;
	REF_IN_Frame_fp = NULL;
	REF_Sys_GetClipboardData_fp = NULL;
	
	memset(&re, 0, sizeof(re));
	reflib_library = NULL;
	reflib_active = false;
}

/*
VID_LoadRefresh
*/
qboolean VID_LoadRefresh(char * name){
	refimport_t ri;
	GetRefAPI_t GetRefAPI;
	char fn[MAX_OSPATH];
	
	/* clean up a previous reflib */
	if(reflib_active){
		re.Shutdown();
		VID_FreeReflib();
		if(KBD_Close_fp)
			KBD_Close_fp();
		if(REF_IN_Shutdown_fp)
			REF_IN_Shutdown_fp();
		KBD_Close_fp = NULL;
		REF_IN_Shutdown_fp = NULL;
	}
	
	Com_Printf("loading %s video driver, ", name);
	
	snprintf(fn, MAX_OSPATH, PKGLIBDIR"/%s", name);
	if((reflib_library = dlopen(fn, RTLD_LAZY)) == 0){
		Com_Printf("failed\n  %s\n", dlerror());
		return false;
	}
	
	Com_Printf("ok\n");
	
	ri.Cmd_AddCommand = Cmd_AddCommand;
	ri.Cmd_RemoveCommand = Cmd_RemoveCommand;
	ri.Cmd_Argc = Cmd_Argc;
	ri.Cmd_Argv = Cmd_Argv;
	ri.Cmd_ExecuteText = Cbuf_ExecuteText;
	ri.Con_Printf = VID_Printf;
	ri.Sys_Error = VID_Error;
	ri.FS_LoadFile = FS_LoadFile;
	ri.FS_FreeFile = FS_FreeFile;
	ri.FS_Gamedir = FS_Gamedir;
	ri.Cvar_Get = Cvar_Get;
	ri.Cvar_Set = Cvar_Set;
	ri.Cvar_SetValue = Cvar_SetValue;
	ri.Vid_NewWindow = VID_NewWindow;
	
	if((GetRefAPI =(GetRefAPI_t) dlsym(reflib_library, "GetRefAPI")) == 0)
		Com_Error(ERR_FATAL, "dlsym failed on %s", name);
		
	re = GetRefAPI(ri);
	
	if(re.api_version != API_VERSION){
		VID_FreeReflib();
		Com_Error(ERR_FATAL, "%s has incompatible api_version", name);
	}
	
	/* Init IN(Mouse) */
	in_state.IN_CenterView_fp = IN_CenterView;
	in_state.Key_Event_fp = Do_Key_Event;
	in_state.viewangles = cl.viewangles;
	in_state.in_strafe_state = &in_strafe.state;
	
	if((REF_IN_Init_fp = (void(*)(in_state_t *)) dlsym(reflib_library, "REF_IN_Init")) == NULL ||
			(REF_IN_Shutdown_fp = (void(*)(void)) dlsym(reflib_library, "REF_IN_Shutdown")) == NULL ||
			(REF_IN_Activate_fp = (void(*)(qboolean)) dlsym(reflib_library, "REF_IN_Activate")) == NULL ||
			(REF_IN_Commands_fp = (void(*)(void)) dlsym(reflib_library, "REF_IN_Commands")) == NULL ||
			(REF_IN_Move_fp = (void(*)(usercmd_t *)) dlsym(reflib_library, "REF_IN_Move")) == NULL ||
			(REF_IN_Frame_fp = (void(*)(void)) dlsym(reflib_library, "REF_IN_Frame")) == NULL)
		Sys_Error("No REF_IN functions in REF.\n");
		
	/* this one is optional */
	REF_Sys_GetClipboardData_fp = (char *(*)(void)) dlsym(reflib_library, "REF_Sys_GetClipboardData");
	
	Real_IN_Init();
	
	if(re.Init(0, 0) == -1){
		re.Shutdown();
		VID_FreeReflib();
		return false;
	}
	
	/* Init KBD */
	if((KBD_Init_fp =(void(*)(Key_Event_fp_t)) dlsym(reflib_library, "KBD_Init")) == NULL ||
			(KBD_Update_fp =(void(*)(void)) dlsym(reflib_library, "KBD_Update")) == NULL ||
			(KBD_Close_fp =(void(*)(void)) dlsym(reflib_library, "KBD_Close")) == NULL)
		Sys_Error("No KBD functions in REF.\n");
		
	KBD_Init_fp(Do_Key_Event);
	
	Key_ClearStates();
	
	reflib_active = true;
	return true;
}

/*
VID_Restart_f

*/
void VID_Restart_f(void){
	char name[100];
	
	vid_ref->modified = false;
	vid_width->modified = false;
	vid_height->modified = false;
	vid_fullscreen->modified = false;
	
	cl.refresh_prepped = false;
	cls.disable_screen = true;
		
	sprintf(name, "ref_%s.so", vid_ref->string);
	if(!VID_LoadRefresh(name)){  //failed
		Com_Error(ERR_FATAL, "couldn't load %s\n", name);
		return;
	}
		
	cls.disable_screen = false;
}

/*
VID_Init
*/
void VID_Init(void){
	
	Com_Printf("\n------- video initialization -------\n");
	
	vid_ref = Cvar_Get("vid_ref", "glx", CVAR_ARCHIVE);
	vid_xpos = Cvar_Get("vid_xpos", "3", CVAR_ARCHIVE);
	vid_ypos = Cvar_Get("vid_ypos", "22", CVAR_ARCHIVE);
	vid_width = Cvar_Get("vid_width", "640", CVAR_ARCHIVE);
	vid_height = Cvar_Get("vid_height", "480", CVAR_ARCHIVE);
	vid_fullscreen = Cvar_Get("vid_fullscreen", "0", CVAR_ARCHIVE);
	vid_gamma = Cvar_Get("vid_gamma", "1", CVAR_ARCHIVE);
	
	/* Add some console commands that we want to handle */
	Cmd_AddCommand("vid_restart", VID_Restart_f);
	
	/* Disable the 3Dfx splash screen */
	putenv("FX_GLIDE_NO_SPLASH=0");
	
	/* Start the graphics mode and load refresher */
	VID_Restart_f();
	
	Com_Printf("\n");
}

/*
VID_Shutdown
*/
void VID_Shutdown(void){
	if(reflib_active){
		if(KBD_Close_fp)
			KBD_Close_fp();
		if(REF_IN_Shutdown_fp)
			REF_IN_Shutdown_fp();
		KBD_Close_fp = NULL;
		REF_IN_Shutdown_fp = NULL;
		re.Shutdown();
		VID_FreeReflib();
	}
}


// This is fake, it's acutally done by the Refresh load
void IN_Init(void){}

void Real_IN_Init(void){
	if(REF_IN_Init_fp)
		REF_IN_Init_fp(&in_state);
}

void IN_Shutdown(void){
	if(REF_IN_Shutdown_fp)
		REF_IN_Shutdown_fp();
}

void IN_Commands(void){
	if(REF_IN_Commands_fp)
		REF_IN_Commands_fp();
}

void IN_Move(usercmd_t *cmd){
	if(REF_IN_Move_fp)
		REF_IN_Move_fp(cmd);
}

void IN_Frame(void){
	if(REF_IN_Activate_fp){
		if(!cl.refresh_prepped || cls.key_dest == key_console)
			REF_IN_Activate_fp(false);
		else
			REF_IN_Activate_fp(true);
	}
	
	if(REF_IN_Frame_fp)
		REF_IN_Frame_fp();
}

void IN_Activate(qboolean active){}

void Do_Key_Event(int key, qboolean down){
	Key_Event(key, down, Sys_Milliseconds());
}

char *Sys_GetClipboardData(void){
	if(REF_Sys_GetClipboardData_fp)
		return REF_Sys_GetClipboardData_fp();
	else
		return NULL;
}

==> quetoo-0.4.0.tar.bz2: quetoo-0.4.0/src/vid_ref.c <==
/*
* Copyright(c) 1997-2001 Id Software, Inc.
* Copyright(c) 2002 The Quakeforge Project.
* Copyright(c) 2006 Quetoo.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or(at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
* 
* See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*
* Main windowed and fullscreen graphics interface module. This module
* is used for both the software and OpenGL rendering versions of the
* Quake refresh engine.
*/

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <dlfcn.h>

#include "client.h"

// structure containing functions exported from refresh
refexport_t re;

/* Console variables that we need to access from this module */
cvar_t *vid_ref;        /* name of refresher dll */
cvar_t *vid_width;		/* window size x */
cvar_t *vid_height;		/* window size y */
cvar_t *vid_fullscreen; /* fullscreen on or off */
cvar_t *vid_gamma;      /* gamma value */

// global video state; used by other modules
viddef_t viddef;

// handle to refresher
static void *reflib_library = NULL;

qboolean reflib_active = 0;

// keyboard
void Do_Key_Event(int key, qboolean down);

void(*KBD_Update_fp)(void);
void(*KBD_Init_fp)(Key_Event_fp_t fp);
void(*KBD_Close_fp)(void);

// mouse
in_state_t in_state;

void(*VID_IN_Init_fp)(in_state_t *in_state_p);
void(*VID_IN_Shutdown_fp)(void);
void(*VID_IN_Activate_fp)(qboolean active);
void(*VID_IN_Commands_fp)(void);
void(*VID_IN_Move_fp)(usercmd_t *cmd);
void(*VID_IN_Frame_fp)(void);

void Real_IN_Init(void);

// clipboard
char *(*VID_Sys_GetClipboardData_fp)(void);

/*
 
REF GLUE
 
*/

#define MAXPRINTMSG	4096
void VID_Printf(int print_level, char *fmt, ...){
	va_list	argptr;
	char msg[MAXPRINTMSG];
	
	va_start(argptr, fmt);
	vsnprintf(msg, MAXPRINTMSG, fmt, argptr);
	va_end(argptr);
	
	if(print_level == PRINT_ALL)
		Com_Printf("%s", msg);
	else
		Com_DPrintf("%s", msg);
}

void __attribute__((noreturn)) VID_Error(int err_level, char *fmt, ...){
	va_list	argptr;
	char msg[MAXPRINTMSG];
	
	va_start(argptr, fmt);
	vsnprintf(msg, MAXPRINTMSG, fmt, argptr);
	va_end(argptr);
	
	Com_Error(err_level, "%s", msg);
}


/*
** VID_NewWindow
*/
void VID_NewWindow(int width, int height){
	viddef.width = width;
	viddef.height = height;
}

void VID_FreeReflib(void){
	if(reflib_library){
		if(KBD_Close_fp)
			KBD_Close_fp();
		if(VID_IN_Shutdown_fp)
			VID_IN_Shutdown_fp();
		dlclose(reflib_library);
	}
	
	KBD_Init_fp = NULL;
	KBD_Update_fp = NULL;
	KBD_Close_fp = NULL;
	VID_IN_Init_fp = NULL;
	VID_IN_Shutdown_fp = NULL;
	VID_IN_Activate_fp = NULL;
	VID_IN_Commands_fp = NULL;
	VID_IN_Move_fp = NULL;
	VID_IN_Frame_fp = NULL;
	VID_Sys_GetClipboardData_fp = NULL;
	
	memset(&re, 0, sizeof(re));
	reflib_library = NULL;
	reflib_active = false;
}

/*
VID_LoadRefresh
*/
qboolean VID_LoadRefresh(char * name){
	refimport_t ri;
	GetRefAPI_t GetRefAPI;
	char fn[MAX_OSPATH];
	
	/* clean up a previous reflib */
	if(reflib_active){
		re.Shutdown();
		VID_FreeReflib();
		if(KBD_Close_fp)
			KBD_Close_fp();
		if(VID_IN_Shutdown_fp)
			VID_IN_Shutdown_fp();
		KBD_Close_fp = NULL;
		VID_IN_Shutdown_fp = NULL;
	}
	
	Com_Printf("loading %s video driver, ", name);
	
	snprintf(fn, MAX_OSPATH, PKGLIBDIR"/%s", name);
	if((reflib_library = dlopen(fn, RTLD_LAZY)) == 0){
		Com_Printf("failed\n  %s\n", dlerror());
		return false;
	}
	
	Com_Printf("ok\n");
	
	ri.Cmd_AddCommand = Cmd_AddCommand;
	ri.Cmd_RemoveCommand = Cmd_RemoveCommand;
	ri.Cmd_Argc = Cmd_Argc;
	ri.Cmd_Argv = Cmd_Argv;
	ri.Cmd_ExecuteText = Cbuf_ExecuteText;
	ri.Con_Printf = VID_Printf;
	ri.Sys_Error = VID_Error;
	ri.FS_LoadFile = FS_LoadFile;
	ri.FS_FreeFile = FS_FreeFile;
	ri.FS_Gamedir = FS_Gamedir;
	ri.Hunk_Begin = Hunk_Begin;
	ri.Hunk_End = Hunk_End;
	ri.Hunk_Alloc = Hunk_Alloc;
	ri.Hunk_FreeAll = Hunk_FreeAll;
	ri.CL_Active = CL_Active;
	ri.Cvar_Get = Cvar_Get;
	ri.Cvar_Set = Cvar_Set;
	ri.Cvar_SetValue = Cvar_SetValue;
	ri.Vid_NewWindow = VID_NewWindow;
	
	if((GetRefAPI =(GetRefAPI_t) dlsym(reflib_library, "GetRefAPI")) == 0)
		Com_Error(ERR_FATAL, "dlsym failed on %s", name);
		
	re = GetRefAPI(ri);
	
	if(re.api_version != VID_API_VERSION){
		VID_FreeReflib();
		Com_Error(ERR_FATAL, "%s has incompatible api_version", name);
	}
	
	/* Init IN */
	in_state.IN_CenterView_fp = IN_CenterView;
	in_state.Key_Event_fp = Do_Key_Event;
	in_state.viewangles = cl.viewangles;
	
	if((VID_IN_Init_fp = (void(*)(in_state_t *)) dlsym(reflib_library, "VID_IN_Init")) == NULL ||
			(VID_IN_Shutdown_fp = (void(*)(void)) dlsym(reflib_library, "VID_IN_Shutdown")) == NULL ||
			(VID_IN_Activate_fp = (void(*)(qboolean)) dlsym(reflib_library, "VID_IN_Activate")) == NULL ||
			(VID_IN_Commands_fp = (void(*)(void)) dlsym(reflib_library, "VID_IN_Commands")) == NULL ||
			(VID_IN_Move_fp = (void(*)(usercmd_t *)) dlsym(reflib_library, "VID_IN_Move")) == NULL ||
			(VID_IN_Frame_fp = (void(*)(void)) dlsym(reflib_library, "VID_IN_Frame")) == NULL)
		Sys_Error("No VID_IN functions in REF.\n");
		
	/* this one is optional */
	VID_Sys_GetClipboardData_fp = (char *(*)(void)) dlsym(reflib_library, "VID_Sys_GetClipboardData");
	
	Real_IN_Init();
	
	if(re.Init(0, 0) == -1){
		re.Shutdown();
		VID_FreeReflib();
		return false;
	}
	
	/* Init KBD */
	if((KBD_Init_fp =(void(*)(Key_Event_fp_t)) dlsym(reflib_library, "KBD_Init")) == NULL ||
			(KBD_Update_fp =(void(*)(void)) dlsym(reflib_library, "KBD_Update")) == NULL ||
			(KBD_Close_fp =(void(*)(void)) dlsym(reflib_library, "KBD_Close")) == NULL)
		Sys_Error("No KBD functions in REF.\n");
		
	KBD_Init_fp(Do_Key_Event);
	
	Key_ClearStates();
	
	reflib_active = true;
	return true;
}

/*
VID_Restart_f

*/
void VID_Restart_f(void){
	char name[16];
	
	vid_ref->modified = false;
	vid_width->modified = false;
	vid_height->modified = false;
	vid_fullscreen->modified = false;
	
	cl.refresh_prepped = false;
	cls.disable_screen = true;
		
	snprintf(name, sizeof(name), "vid_%s.so", vid_ref->string);
	if(!VID_LoadRefresh(name)){  //failed
		Com_Error(ERR_FATAL, "couldn't load %s\n", name);
		return;
	}
		
	cls.disable_screen = false;
}

/*
VID_Init
*/
void VID_Init(void){
	
	Com_Printf("\n------- video initialization -------\n");
	
	vid_ref = Cvar_Get("vid_ref", "glx", CVAR_ARCHIVE);
	vid_width = Cvar_Get("vid_width", "640", CVAR_ARCHIVE);
	vid_height = Cvar_Get("vid_height", "480", CVAR_ARCHIVE);
	vid_fullscreen = Cvar_Get("vid_fullscreen", "1", CVAR_ARCHIVE);
	vid_gamma = Cvar_Get("vid_gamma", "1", CVAR_ARCHIVE);
	
	/* Add some console commands that we want to handle */
	Cmd_AddCommand("vid_restart", VID_Restart_f);
	
	/* Disable the 3Dfx splash screen */
	putenv("FX_GLIDE_NO_SPLASH=0");
	
	/* Start the graphics mode and load refresher */
	VID_Restart_f();
	
	Com_Printf("\n");
}

/*
VID_Shutdown
*/
void VID_Shutdown(void){
	if(reflib_active){
		if(KBD_Close_fp)
			KBD_Close_fp();
		if(VID_IN_Shutdown_fp)
			VID_IN_Shutdown_fp();
		KBD_Close_fp = NULL;
		VID_IN_Shutdown_fp = NULL;
		re.Shutdown();
		VID_FreeReflib();
	}
}


// This is fake, it's acutally done by the Refresh load
void IN_Init(void){}

void Real_IN_Init(void){
	if(VID_IN_Init_fp)
		VID_IN_Init_fp(&in_state);
}

void IN_Shutdown(void){
	if(VID_IN_Shutdown_fp)
		VID_IN_Shutdown_fp();
}

void IN_Commands(void){
	if(VID_IN_Commands_fp)
		VID_IN_Commands_fp();
}

void IN_Move(usercmd_t *cmd){
	if(VID_IN_Move_fp)
		VID_IN_Move_fp(cmd);
}

void IN_Frame(void){
	if(VID_IN_Activate_fp){
		if(!cl.refresh_prepped || cls.key_dest == key_console)
			VID_IN_Activate_fp(false);
		else
			VID_IN_Activate_fp(true);
	}
	
	if(VID_IN_Frame_fp)
		VID_IN_Frame_fp();
}

void IN_Activate(qboolean active){}

void Do_Key_Event(int key, qboolean down){
	Key_Event(key, down, Sys_Milliseconds());
}

char *Sys_GetClipboardData(void){
	if(VID_Sys_GetClipboardData_fp)
		return VID_Sys_GetClipboardData_fp();
	else
		return NULL;
}
